# SNFtool Simulation tutorial
 Here we will go through an example for running SimilarityNetworkFusion on a simulated data.
 The data was generated by paper's authors and the details appears in the
 supplementary data, look for the second simulation. In short, there are two
 data sets representing 200 samples from two classes. In the first data set
 the first class have a Normal noise added, and the second class a Gamma
 distributed noise. In the second data set the opposite.

 This tutorial was adopted from the Readme of the SNFtool R package on github:
 https://github.com/cran/SNFtool

 ```julia
## Install the package
Pkg.add("SimilarityNetworkFusion")
using SimilarityNetworkFusion
using DataFrames

## First, set all the parameters:
K = 20;		# number of neighbors, usually (10~30)
alpha = 0.5;  	# hyperparameter, usually (0.3~0.8)
T = 10; 	# Number of Iterations, usually (10~20)

## Data is an array of 3 data matrices
## outcomes is a matrix with 3 columns: sample id, survival rate and class
Data, outcomes = dataset("Lung")
## We will focus here on the class prediction. For survival rate, see elsewhere.
truelabels = outcomes[:,3]

## If the data are all continuous values, we recommend the users to perform standard normalization before using SNF, though it is optional depending on the data the users want to use.  
Data = [sapply_df(DataFrame(d),standardize) for d in Data]

## Calculate the pair-wise distance; If the data is continuous, we recommend to use the function "dist2" as follows; if the data is discrete, we recommend the users to use ???""
Dist = [dist2(d) for d in Data]

## next, construct similarity graphs
W = [affinityMatrix(dist, K, alpha) for dist in Dist]

## These similarity graphs have complementary information about clusters.
# Plot each affinity matrix with same order of samples
Gadfly.hstack([displayClusters(w,truelabels) for w in W])

## next, we fuse all the graphs
## then the overall matrix can be computed by similarity network fusion(SNF):
WSNF = SNF(W, K, T)

## With this unified graph W of size n x n, you can do either spectral clustering or Kernel NMF. If you need help with further clustering, please let us know.
## for example, spectral clustering

C = 2 					# number of clusters
group = spectralClustering(WSNF, C); 	# the final subtypes information

## you can evaluate the goodness of the obtained clustering results by calculate Normalized mutual information (NMI): if NMI is close to 1, it indicates that the obtained clustering is very close to the "true" cluster information; if NMI is close to 0, it indicates the obtained clustering is not similar to the "true" cluster information.

displayClusters(WSNF, group)
# SNFNMI = calNMI(group, truelabels)

## you can also find the concordance between each individual network and the fused network

#ConcordanceMatrix = concordanceNetworkNMI(list(W, W1,W2));
```

```@docs
SimilarityNetworkFusion.dataset
SimilarityNetworkFusion.SNF
```
